package de.intevation.flys.client.server.meta;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import de.intevation.flys.client.shared.model.DataCageTree;
import de.intevation.flys.client.shared.model.DataCageNode;

import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;

import de.intevation.artifacts.common.utils.XMLUtils;

public class Converter
{
    public interface NodeConverter
    {
        DataCageNode convert(Element node, Converter converter);

    } // interface NodeConverter

    public static class NameConverter implements NodeConverter {
        public DataCageNode convert(Element node, Converter converter) {
            System.err.println("NameConverter called");
            DataCageNode out = new DataCageNode(node.getAttribute("name"));
            converter.convertChildren(out, node);
            return out;
        }
    } // class NameConverter

    public static class I18NConverter implements NodeConverter {
        public DataCageNode convert(Element node, Converter converter) {
            System.err.println("I18NConverter called");
            DataCageNode out =
                new DataCageNode("${" + node.getLocalName() + "}");
            converter.convertChildren(out, node);
            return out;
        }
    } // I18NConverter

    protected Map<String, NodeConverter> converters;

    protected void convertChildren(DataCageNode parent, Element sub) {
        System.err.println("convertChildren called");
        NodeList children = sub.getChildNodes();
        for (int i = 0, N = children.getLength(); i < N; ++i) {
            Node child = children.item(i);
            if (child.getNodeType() == Node.ELEMENT_NODE) {
                String name = child.getLocalName();
                NodeConverter converter = converters.get(name);
                if (converter == null) {
                    converter = I18N_CONVERTER;
                }
                DataCageNode son = converter.convert(
                    (Element)child, this);
                parent.addChild(son);
            }
        } // for all children
    }

    public Converter() {
        converters = new HashMap<String, NodeConverter>();
        registerConverters();
    }

    public static final NodeConverter NAME_CONVERTER = new NameConverter(); 
    public static final NodeConverter I18N_CONVERTER = new I18NConverter(); 

    protected void registerConverters() {
        System.err.println("register converters called");
        converters.put("datacage",   NAME_CONVERTER);
        converters.put("river",      NAME_CONVERTER);
        converters.put("gauge",      NAME_CONVERTER);
        converters.put("historical", NAME_CONVERTER);
        converters.put("column",     NAME_CONVERTER);
    }

    public DataCageTree convert(Document document) {
        System.err.println("convert called");
        //System.err.println(XMLUtils.toString(document));

        ArrayList<DataCageNode> roots = new ArrayList<DataCageNode>();
        NodeList nodes = document.getChildNodes();
        for (int i = 0, N = nodes.getLength(); i < N; ++i) {
            Node node = nodes.item(i);
            if (node.getNodeType() == Node.ELEMENT_NODE) {
                System.err.println("search for name: " + node.getLocalName());
                NodeConverter converter = converters.get(node.getLocalName());
                if (converter == null) {
                    converter = I18N_CONVERTER;
                }
                roots.add(converter.convert((Element)node, this));
            }
        }
        return roots.isEmpty()
            ? new DataCageTree()
            : new DataCageTree(roots.get(0));
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
